@@ -1,5 +1,8 @@
+mod external_data_readers;
+
+pub use external_data_readers::ExternalDataReaders;
+
 use std::{
-    collections::HashMap,
     convert::TryFrom,
     error, fmt,
     io::{self, BufRead, Read},
@@ -53,7 +56,7 @@ where
 {
     compression_header: &'a CompressionHeader,
     core_data_reader: BitReader<CDR>,
-    external_data_readers: HashMap<Itf8, EDR>,
+    external_data_readers: ExternalDataReaders<EDR>,
     reference_sequence_id: ReferenceSequenceId,
     prev_alignment_start: Option<sam::record::Position>,
 }
@@ -66,7 +69,7 @@ where
     pub fn new(
         compression_header: &'a CompressionHeader,
         core_data_reader: BitReader<CDR>,
-        external_data_readers: HashMap<Itf8, EDR>,
+        external_data_readers: ExternalDataReaders<EDR>,
         reference_sequence_id: ReferenceSequenceId,
         initial_alignment_start: Option<sam::record::Position>,
     ) -> Self {
@@ -899,7 +902,7 @@ where
 fn decode_byte<CDR, EDR>(
     encoding: &Encoding,
     core_data_reader: &mut BitReader<CDR>,
-    external_data_readers: &mut HashMap<Itf8, EDR>,
+    external_data_readers: &mut ExternalDataReaders<EDR>,
 ) -> io::Result<u8>
 where
     CDR: Read,
@@ -932,7 +935,7 @@ where
 fn decode_itf8<CDR, EDR>(
     encoding: &Encoding,
     core_data_reader: &mut BitReader<CDR>,
-    external_data_readers: &mut HashMap<Itf8, EDR>,
+    external_data_readers: &mut ExternalDataReaders<EDR>,
 ) -> io::Result<Itf8>
 where
     CDR: Read,
@@ -963,7 +966,7 @@ where
 fn decode_byte_array<CDR, EDR>(
     encoding: &Encoding,
     core_data_reader: &mut BitReader<CDR>,
-    external_data_readers: &mut HashMap<Itf8, EDR>,
+    external_data_readers: &mut ExternalDataReaders<EDR>,
     buf: Option<Vec<u8>>,
 ) -> io::Result<Vec<u8>>
 where
