@@ -5,8 +5,8 @@ use std::convert::TryFrom;
 use noodles_fasta as fasta;
 use noodles_sam::record::Cigar;
 
-use crate::data_container::compression_header::{
-    preservation_map::substitution_matrix::Base, SubstitutionMatrix,
+use crate::data_container::{
+    compression_header::preservation_map::substitution_matrix::Base, CompressionHeader,
 };
 
 use super::Feature;
@@ -14,7 +14,7 @@ use super::Feature;
 /// Resolves the read bases.
 pub fn resolve_bases(
     reference_sequence_record: &fasta::Record,
-    substitution_matrix: &SubstitutionMatrix,
+    compression_header: &CompressionHeader,
     features: &[Feature],
     alignment_start: i32,
     read_len: usize,
@@ -25,6 +25,7 @@ pub fn resolve_bases(
     let mut read_pos = 0;
 
     let reference_sequence = reference_sequence_record.sequence();
+    let substitution_matrix = compression_header.preservation_map().substitution_matrix();
 
     for feature in features {
         let feature_pos = feature.position() as usize;
@@ -109,10 +110,15 @@ pub fn resolve_features(features: &[Feature], read_len: i32) -> Cigar {
         let op = Op::new(kind, len as u32);
         ops.push(op);
 
-        i += len;
+        if matches!(
+            kind,
+            Kind::Match | Kind::Insertion | Kind::SoftClip | Kind::SeqMatch | Kind::SeqMismatch
+        ) {
+            i += len;
+        }
     }
 
-    if i < read_len {
+    if i <= read_len {
         let len = read_len - i + 1;
         let op = Op::new(Kind::Match, len as u32);
         ops.push(op);
@@ -147,6 +153,27 @@ mod tests {
             Cigar::from(vec![Op::new(Kind::Match, 3), Op::new(Kind::SoftClip, 1)])
         );
 
+        let features = [Feature::HardClip(1, 2)];
+        assert_eq!(
+            resolve_features(&features, 4),
+            Cigar::from(vec![Op::new(Kind::HardClip, 2), Op::new(Kind::Match, 4)]),
+        );
+
+        // FIXME
+        let features = [
+            Feature::SoftClip(1, b"A".to_vec()),
+            Feature::Substitution(3, 0),
+        ];
+        assert_eq!(
+            resolve_features(&features, 4),
+            Cigar::from(vec![
+                Op::new(Kind::SoftClip, 1),
+                Op::new(Kind::Match, 1),
+                Op::new(Kind::Match, 1),
+                Op::new(Kind::Match, 1),
+            ])
+        );
+
         // FIXME
         let features = [Feature::Substitution(2, 0)];
         assert_eq!(
